import { stringify } from 'yaml';
import { promises as fs } from 'fs';
import { homedir } from 'os';
import { join } from 'path';
import chalk from 'chalk';
import { createTask, createSubtask, getListStatuses, getSpaceTags, createTag, manageStatus } from './clickup.js';
import { getConfig } from '../config/store.js';
const CONFIG_DIR = join(homedir(), '.task-cli');
const BACKUP_DIR = join(CONFIG_DIR, 'backups');
export async function exportConfig(type) {
    const config = await getConfig();
    const spaceId = config.clickup.defaultSpace;
    const listId = config.clickup.defaultList;
    if (!spaceId || !listId) {
        throw new Error('Default space and list must be configured');
    }
    const exportData = {};
    if (type === 'tags' || type === 'all') {
        const tags = await getSpaceTags(spaceId);
        exportData.tags = tags.map(tag => ({
            name: tag.name,
            bg_color: tag.tag_bg,
            fg_color: tag.tag_fg
        }));
    }
    if (type === 'status' || type === 'all') {
        const statuses = await getListStatuses(listId);
        exportData.statuses = statuses.map(status => ({
            name: status.status,
            color: status.color || '#000000',
            order: status.orderindex || 0
        }));
    }
    return stringify(exportData);
}
function validateConfig(config) {
    const errors = [];
    if (!config) {
        errors.push({
            type: 'config',
            item: 'root',
            message: 'Configuration is empty'
        });
        return errors;
    }
    // Validate tags
    if (config.tags) {
        for (const [index, tag] of config.tags.entries()) {
            if (!tag.name) {
                errors.push({
                    type: 'tag',
                    item: `tags[${index}]`,
                    message: 'Tag name is required'
                });
            }
            if (!tag.bg_color) {
                errors.push({
                    type: 'tag',
                    item: `tags[${index}]`,
                    message: 'Background color is required'
                });
            }
            if (!tag.fg_color) {
                errors.push({
                    type: 'tag',
                    item: `tags[${index}]`,
                    message: 'Foreground color is required'
                });
            }
        }
    }
    // Validate tasks
    if (config.tasks) {
        for (const [index, task] of config.tasks.entries()) {
            if (!task.name) {
                errors.push({
                    type: 'task',
                    item: `tasks[${index}]`,
                    message: 'Task name is required'
                });
            }
            if (!task.priority || ![1, 2, 3, 4].includes(task.priority)) {
                errors.push({
                    type: 'task',
                    item: `tasks[${index}]`,
                    message: 'Priority must be 1 (urgent), 2 (high), 3 (normal), or 4 (low)'
                });
            }
            if (!task.status) {
                errors.push({
                    type: 'task',
                    item: `tasks[${index}]`,
                    message: 'Status is required'
                });
            }
            if (task.due_date && !/^\d{4}-\d{2}-\d{2}$/.test(task.due_date)) {
                errors.push({
                    type: 'task',
                    item: `tasks[${index}]`,
                    message: 'Due date must be in YYYY-MM-DD format'
                });
            }
            // Validate subtasks if present
            if (task.subtasks) {
                for (const [subtaskIndex, subtask] of task.subtasks.entries()) {
                    if (!subtask.name) {
                        errors.push({
                            type: 'task',
                            item: `tasks[${index}].subtasks[${subtaskIndex}]`,
                            message: 'Subtask name is required'
                        });
                    }
                    if (subtask.priority && ![1, 2, 3, 4].includes(subtask.priority)) {
                        errors.push({
                            type: 'task',
                            item: `tasks[${index}].subtasks[${subtaskIndex}]`,
                            message: 'Priority must be 1 (urgent), 2 (high), 3 (normal), or 4 (low)'
                        });
                    }
                    if (subtask.due_date && !/^\d{4}-\d{2}-\d{2}$/.test(subtask.due_date)) {
                        errors.push({
                            type: 'task',
                            item: `tasks[${index}].subtasks[${subtaskIndex}]`,
                            message: 'Due date must be in YYYY-MM-DD format'
                        });
                    }
                }
            }
        }
    }
    return errors;
}
async function applyTaskChanges(tasks, dryRun = false) {
    const config = await getConfig();
    const listId = config.clickup.defaultList;
    if (!listId) {
        throw new Error('Default list must be configured');
    }
    for (const task of tasks) {
        if (dryRun) {
            console.log(chalk.blue(`Would create task: ${task.name}`));
            if (task.description)
                console.log(chalk.blue(`  Description: ${task.description}`));
            console.log(chalk.blue(`  Priority: ${task.priority}`));
            console.log(chalk.blue(`  Status: ${task.status}`));
            if (task.tags)
                console.log(chalk.blue(`  Tags: ${task.tags.join(', ')}`));
            if (task.subtasks) {
                console.log(chalk.blue(`  Subtasks:`));
                for (const subtask of task.subtasks) {
                    console.log(chalk.blue(`    - ${subtask.name}`));
                    if (subtask.description)
                        console.log(chalk.blue(`      Description: ${subtask.description}`));
                    if (subtask.priority)
                        console.log(chalk.blue(`      Priority: ${subtask.priority}`));
                    if (subtask.status)
                        console.log(chalk.blue(`      Status: ${subtask.status}`));
                    if (subtask.tags)
                        console.log(chalk.blue(`      Tags: ${subtask.tags.join(', ')}`));
                }
            }
            continue;
        }
        try {
            const createdTask = await createTask(listId, task.name, task.description || '', task.priority, task.status);
            console.log(chalk.green(`Created task: ${task.name}`));
            // Create subtasks if present
            if (task.subtasks && task.subtasks.length > 0) {
                for (const subtask of task.subtasks) {
                    try {
                        await createSubtask(createdTask.id, subtask.name, subtask.description, subtask.priority, subtask.status);
                        console.log(chalk.green(`  Created subtask: ${subtask.name}`));
                    }
                    catch (error) {
                        console.error(chalk.red(`  Failed to create subtask ${subtask.name}:`), error);
                    }
                }
            }
        }
        catch (error) {
            console.error(chalk.red(`Failed to create task ${task.name}:`), error);
        }
    }
}
async function applyTagChanges(spaceId, tags) {
    for (const tag of tags) {
        try {
            await createTag(spaceId, tag.name, tag.bg_color, tag.fg_color);
            console.log(`Created tag: ${tag.name}`);
        }
        catch (error) {
            console.error(`Failed to create tag "${tag.name}":`, error);
        }
    }
}
async function applyStatusChanges(listId, statuses, dryRun = false) {
    if (dryRun) {
        console.log(chalk.blue('\nStatus changes to be applied:'));
        for (const status of statuses) {
            console.log(`  - ${status.name} (${status.color}, order: ${status.order})`);
        }
        return;
    }
    console.log(chalk.blue('\nApplying status changes...'));
    for (const status of statuses) {
        try {
            await manageStatus(listId, status.name, status.color, status.order);
            console.log(chalk.green(`  ✓ Managed status: ${status.name}`));
        }
        catch (error) {
            if (error instanceof Error) {
                console.error(chalk.red(`  ✗ ${error.message}`));
            }
            else {
                console.error(chalk.red(`  ✗ Failed to manage status ${status.name}`));
            }
        }
    }
}
export async function applyConfig(config, dryRun = false) {
    // Validate configuration
    const errors = validateConfig(config);
    if (errors.length > 0) {
        console.error(chalk.red('\nConfiguration validation failed:'));
        for (const error of errors) {
            console.error(`  - ${error.message}`);
        }
        throw new Error('Invalid configuration');
    }
    // Create backup
    if (!dryRun) {
        await createBackup();
    }
    const clickupConfig = await getConfig();
    const spaceId = clickupConfig.clickup.defaultSpace;
    const listId = clickupConfig.clickup.defaultList;
    if (!spaceId || !listId) {
        throw new Error('Default space and list must be configured');
    }
    // Apply changes
    if (config.tags) {
        await applyTagChanges(spaceId, config.tags);
    }
    if (config.statuses) {
        await applyStatusChanges(listId, config.statuses, dryRun);
    }
    if (config.tasks) {
        await applyTaskChanges(config.tasks, dryRun);
    }
}
async function createBackup() {
    try {
        const timestamp = new Date().toISOString().replace(/[:.]/g, '-');
        const backupFile = join(BACKUP_DIR, `config-${timestamp}.yml`);
        // Create backup directory if it doesn't exist
        await fs.mkdir(BACKUP_DIR, { recursive: true });
        // Export current config and save to backup file
        const config = await exportConfig('all');
        await fs.writeFile(backupFile, config);
        console.log(chalk.green(`Created backup: ${backupFile}`));
    }
    catch (error) {
        console.error(chalk.yellow('Warning: Failed to create backup'), error);
    }
}
